﻿using System;
using System.Collections.Generic;

namespace EsempioFlyweight
{
    public static class FlyweightFactory
    {
        private static readonly Dictionary<string, IFlyweight> _flyweights =
            new Dictionary<string, IFlyweight>();

        public static IFlyweight GetFlyweight(string name)
        {
            if (!_flyweights.ContainsKey(name))
            {
                _flyweights.Add(name, CreateInstance(name));
            }
            return _flyweights[name];
        }

        public static IEnumerable<IFlyweight> GetFlyweights()
        {
            return _flyweights.Values;
        }

        private static IFlyweight CreateInstance(string name)
        {
            switch (name)
            {
                case "folder":
                    return new ConcreteFlyweight(name, 16, 16);
                case "txtDocument":
                    return new ConcreteFlyweight(name, 24, 24);
                case "datDocument":
                    return new ConcreteFlyweight(name, 32, 32);
                case "exeDocument":
                    return new ConcreteFlyweight(name, 48, 48);
                default:
                    throw new ArgumentException(name);
            }
        }

        private class ConcreteFlyweight : IFlyweight  //  Classe annidata!
        {
            private readonly string _name;
            private readonly int _width;
            private readonly int _height;

            public ConcreteFlyweight(string name, int width, int height)
            {
                _name = name;
                _width = width;
                _height = height;
            }

            #region IFlyweight Members

            public string Name
            {
                get { return _name; }
            }

            public int Width
            {
                get { return _width; }
            }

            public int Height
            {
                get { return _height; }
            }

            public void Draw(int x, int y)
            {
                Console.WriteLine(" {0}: upper left ({1},{2}) - lower right ({3},{4})",
                    Name, x, y, x + Width, y + Height);
            }

            #endregion

            public override string ToString()
            {
                return Name + " (" + Width.ToString() + ", " + Height.ToString() + ")";
            }
        }
    }
}
